@@ -7,6 +7,7 @@
 #include <fcntl.h>
 #include <stdlib.h>
 #include <stdio.h>
+#include <locale.h>
 
 #include <glib.h>
 #include <gio/gio.h>
@@ -415,7 +416,6 @@ main (int argc, char *argv[])
   GInetAddress *inet;
   GSocketAddress *address;
   GSocketService *listener;
-  char *path, *basename;
   char *http_address = NULL;
   char *unixsocket_address = NULL;
   int http_port = 0;
@@ -434,6 +434,8 @@ main (int argc, char *argv[])
     { NULL }
   };
 
+  setlocale (LC_ALL, "");
+
   context = g_option_context_new ("[:DISPLAY] - broadway display daemon");
   g_option_context_add_main_entries (context, entries, GETTEXT_PACKAGE);
   if (!g_option_context_parse (context, &argc, &argv, &error))
@@ -475,6 +477,8 @@ main (int argc, char *argv[])
 #ifdef G_OS_UNIX
   else if (display[0] == ':' && g_ascii_isdigit(display[1]))
     {
+      char *path, *basename;
+
       port = strtol (display + strlen (":"), NULL, 10);
       basename = g_strdup_printf ("broadway%d.socket", port + 1);
       path = g_build_filename (g_get_user_runtime_dir (), basename, NULL);
