@@ -285,11 +285,11 @@ gdk_drag_context_class_init (GdkDragContextClass *klass)
   signals[CANCEL] =
     g_signal_new ("cancel",
                   G_TYPE_FROM_CLASS (object_class),
-                  G_SIGNAL_RUN_FIRST,
+                  G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (GdkDragContextClass, cancel),
                   NULL, NULL,
-                  g_cclosure_marshal_VOID__VOID,
-                  G_TYPE_NONE, 0);
+                  g_cclosure_marshal_VOID__ENUM,
+                  G_TYPE_NONE, 1, GDK_TYPE_DRAG_CANCEL_REASON);
 
   /**
    * GdkDragContext::drop-performed:
@@ -641,6 +641,10 @@ gdk_drag_context_set_hotspot (GdkDragContext *context,
  * be the last call before dropping the reference to the
  * @context.
  *
+ * The #GdkDragContext will only take the first gdk_drag_drop_done()
+ * call as effective, if this function is called multiple times,
+ * all subsequent calls will be ignored.
+ *
  * Since: 3.20
  */
 void
@@ -649,6 +653,11 @@ gdk_drag_drop_done (GdkDragContext *context,
 {
   g_return_if_fail (GDK_IS_DRAG_CONTEXT (context));
 
+  if (context->drop_done)
+    return;
+
+  context->drop_done = TRUE;
+
   if (GDK_DRAG_CONTEXT_GET_CLASS (context)->drop_done)
     GDK_DRAG_CONTEXT_GET_CLASS (context)->drop_done (context, success);
 }
@@ -704,11 +713,12 @@ gdk_drag_context_set_cursor (GdkDragContext *context,
 }
 
 void
-gdk_drag_context_cancel (GdkDragContext *context)
+gdk_drag_context_cancel (GdkDragContext      *context,
+                         GdkDragCancelReason  reason)
 {
   g_return_if_fail (GDK_IS_DRAG_CONTEXT (context));
 
-  g_signal_emit (context, signals[CANCEL], 0);
+  g_signal_emit (context, signals[CANCEL], 0, reason);
 }
 
 GList *
