@@ -80,7 +80,7 @@ create_pointer (GdkDeviceManager *device_manager,
                        "input-source", GDK_SOURCE_MOUSE,
                        "input-mode", GDK_MODE_SCREEN,
                        "has-cursor", type == GDK_DEVICE_TYPE_MASTER,
-                       "display", _gdk_display,
+                       "display", gdk_device_manager_get_display (device_manager),
                        "device-manager", device_manager,
                        NULL);
 }
@@ -97,7 +97,7 @@ create_keyboard (GdkDeviceManager *device_manager,
                        "input-source", GDK_SOURCE_KEYBOARD,
                        "input-mode", GDK_MODE_SCREEN,
                        "has-cursor", FALSE,
-                       "display", _gdk_display,
+                       "display", gdk_device_manager_get_display (device_manager),
                        "device-manager", device_manager,
                        NULL);
 }
@@ -575,7 +575,7 @@ _gdk_input_wintab_init_check (GdkDeviceManager *_device_manager)
                                  "input-source", GDK_SOURCE_PEN,
                                  "input-mode", GDK_MODE_SCREEN,
                                  "has-cursor", lc.lcOptions & CXO_SYSTEM,
-                                 "display", _gdk_display,
+                                 "display", display,
                                  "device-manager", device_manager,
                                  NULL);
 
@@ -856,17 +856,14 @@ get_modifier_key_state (void)
 }
 
 static GdkDeviceWintab *
-_gdk_device_manager_find_wintab_device (HCTX hctx,
-                                        UINT cursor)
+gdk_device_manager_find_wintab_device (GdkDeviceManagerWin32 *device_manager,
+                                       HCTX                   hctx,
+                                       UINT                   cursor)
 {
-  GdkDeviceManagerWin32 *device_manager;
   GdkDeviceWintab *device;
   GList *tmp_list;
 
-  device_manager = GDK_DEVICE_MANAGER_WIN32 (gdk_display_get_device_manager (_gdk_display));
-  tmp_list = device_manager->wintab_devices;
-
-  while (tmp_list)
+  for (tmp_list = device_manager->wintab_devices; tmp_list; tmp_list = tmp_list->next)
     {
       device = tmp_list->data;
       tmp_list = tmp_list->next;
@@ -880,9 +877,10 @@ _gdk_device_manager_find_wintab_device (HCTX hctx,
 }
 
 gboolean
-_gdk_input_other_event (GdkEvent  *event,
-                        MSG       *msg,
-                        GdkWindow *window)
+gdk_input_other_event (GdkDisplay *display,
+                       GdkEvent   *event,
+                       MSG        *msg,
+                       GdkWindow  *window)
 {
   GdkDeviceManagerWin32 *device_manager;
   GdkDeviceWintab *source_device = NULL;
@@ -903,11 +901,13 @@ _gdk_input_other_event (GdkEvent  *event,
 
   if (event->any.window != wintab_window)
     {
-      g_warning ("_gdk_input_other_event: not wintab_window?");
+      g_warning ("gdk_input_other_event: not wintab_window?");
       return FALSE;
     }
 
-  device_manager = GDK_DEVICE_MANAGER_WIN32 (gdk_display_get_device_manager (_gdk_display));
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS;
+  device_manager = GDK_DEVICE_MANAGER_WIN32 (gdk_display_get_device_manager (display));
+G_GNUC_END_IGNORE_DEPRECATIONS;
   window = gdk_device_get_window_at_position (device_manager->core_pointer, &x, &y);
   if (window == NULL)
     window = gdk_get_default_root_window ();
@@ -915,7 +915,7 @@ _gdk_input_other_event (GdkEvent  *event,
   g_object_ref (window);
 
   GDK_NOTE (EVENTS_OR_INPUT,
-	    g_print ("_gdk_input_other_event: window=%p %+d%+d\n",
+	    g_print ("gdk_input_other_event: window=%p %+d%+d\n",
                GDK_WINDOW_HWND (window), x, y));
 
   if (msg->message == WT_PACKET || msg->message == WT_CSRCHANGE)
@@ -936,14 +936,15 @@ _gdk_input_other_event (GdkEvent  *event,
           return FALSE;
         }
 
-      if ((source_device = _gdk_device_manager_find_wintab_device ((HCTX) msg->lParam,
-								   packet.pkCursor)) == NULL)
+      if ((source_device = gdk_device_manager_find_wintab_device (device_manager,
+                                                                  (HCTX) msg->lParam,
+                                                                  packet.pkCursor)) == NULL)
         return FALSE;
 
       if (gdk_device_get_mode (GDK_DEVICE (source_device)) == GDK_MODE_DISABLED)
         return FALSE;
 
-      last_grab = _gdk_display_get_last_device_grab (_gdk_display, GDK_DEVICE (source_device));
+      last_grab = _gdk_display_get_last_device_grab (display, GDK_DEVICE (source_device));
 
       if (last_grab && last_grab->window)
         {
@@ -1114,8 +1115,9 @@ _gdk_input_other_event (GdkEvent  *event,
       return TRUE;
 
     case WT_CSRCHANGE:
-      if ((source_device = _gdk_device_manager_find_wintab_device ((HCTX) msg->lParam,
-								   packet.pkCursor)) == NULL)
+      if ((source_device = gdk_device_manager_find_wintab_device (device_manager,
+                                                                  (HCTX) msg->lParam,
+                                                                  packet.pkCursor)) == NULL)
         return FALSE;
 
       if (gdk_device_get_mode (GDK_DEVICE (source_device)) == GDK_MODE_DISABLED)
