@@ -20,6 +20,7 @@
 #include "gdk.h"
 #include "gdkprivate-win32.h"
 #include "gdkdisplay-win32.h"
+#include "gdkdevicemanager-win32.h"
 #include "gdkglcontext-win32.h"
 #include "gdkwin32display.h"
 #include "gdkwin32screen.h"
@@ -28,6 +29,37 @@
 
 static int debug_indent = 0;
 
+static GList *
+gdk_win32_display_list_devices (GdkDisplay *display)
+{
+  g_return_val_if_fail (display == gdk_display_get_default (), NULL);
+
+  return GDK_WIN32_DISPLAY (display)->input_devices;
+}
+
+static void
+_gdk_input_init (GdkDisplay *display)
+{
+  GdkDeviceManagerWin32 *device_manager;
+  GList *devices;
+
+  _gdk_input_ignore_core = FALSE;
+
+  device_manager = g_object_new (GDK_TYPE_DEVICE_MANAGER_WIN32,
+                                 "display", display,
+                                 NULL);
+  display->device_manager = GDK_DEVICE_MANAGER (device_manager);
+
+  display->core_pointer = device_manager->core_pointer;
+
+  devices = g_list_append (NULL, display->core_pointer);
+  devices = g_list_concat (devices, g_list_copy (device_manager->wintab_devices));
+
+  GDK_WIN32_DISPLAY (display)->input_devices = devices;
+
+  _gdk_input_wintab_init_check (GDK_DEVICE_MANAGER (device_manager));
+}
+
 /**
  * gdk_win32_display_set_cursor_theme:
  * @display: (type GdkWin32Display): a #GdkDisplay
@@ -129,121 +161,6 @@ gdk_win32_display_get_next_serial (GdkDisplay *display)
 	return 0;
 }
 
-static BOOL CALLBACK
-count_monitor (HMONITOR hmonitor,
-	       HDC      hdc,
-	       LPRECT   rect,
-	       LPARAM   data)
-{
-  gint *n = (gint *) data;
-
-  (*n)++;
-
-  return TRUE;
-}
-
-static BOOL CALLBACK
-enum_monitor (HMONITOR hmonitor,
-	      HDC      hdc,
-	      LPRECT   rect,
-	      LPARAM   data)
-{
-  /* The struct MONITORINFOEX definition is for some reason different
-   * in the winuser.h bundled with mingw64 from that in MSDN and the
-   * official 32-bit mingw (the MONITORINFO part is in a separate "mi"
-   * member). So to keep this easily compileable with either, repeat
-   * the MSDN definition it here.
-   */
-  typedef struct tagMONITORINFOEXA2 {
-    DWORD cbSize;
-    RECT  rcMonitor;
-    RECT  rcWork;
-    DWORD dwFlags;
-    CHAR szDevice[CCHDEVICENAME];
-  } MONITORINFOEXA2;
-
-  MONITORINFOEXA2 monitor_info;
-  HDC hDC;
-
-  gint *index = (gint *) data;
-  GdkWin32Monitor *monitor;
-
-  g_assert (*index < _gdk_num_monitors);
-
-  monitor = _gdk_monitors + *index;
-
-  monitor_info.cbSize = sizeof (MONITORINFOEXA2);
-  GetMonitorInfoA (hmonitor, (MONITORINFO *) &monitor_info);
-
-#ifndef MONITORINFOF_PRIMARY
-#define MONITORINFOF_PRIMARY 1
-#endif
-
-  monitor->name = g_strdup (monitor_info.szDevice);
-  hDC = CreateDCA ("DISPLAY", monitor_info.szDevice, NULL, NULL);
-  monitor->width_mm = GetDeviceCaps (hDC, HORZSIZE);
-  monitor->height_mm = GetDeviceCaps (hDC, VERTSIZE);
-  DeleteDC (hDC);
-  monitor->rect.x = monitor_info.rcMonitor.left;
-  monitor->rect.y = monitor_info.rcMonitor.top;
-  monitor->rect.width = monitor_info.rcMonitor.right - monitor_info.rcMonitor.left;
-  monitor->rect.height = monitor_info.rcMonitor.bottom - monitor_info.rcMonitor.top;
-
-  if (monitor_info.dwFlags & MONITORINFOF_PRIMARY &&
-      *index != 0)
-    {
-      /* Put primary monitor at index 0, just in case somebody needs
-       * to know which one is the primary.
-       */
-      GdkWin32Monitor temp = *monitor;
-      *monitor = _gdk_monitors[0];
-      _gdk_monitors[0] = temp;
-    }
-
-  (*index)++;
-
-  return TRUE;
-}
-
-void
-_gdk_monitor_init (void)
-{
-  gint i, index;
-
-  _gdk_num_monitors = 0;
-
-  EnumDisplayMonitors (NULL, NULL, count_monitor, (LPARAM) &_gdk_num_monitors);
-
-  _gdk_monitors = g_renew (GdkWin32Monitor, _gdk_monitors, _gdk_num_monitors);
-
-  index = 0;
-  EnumDisplayMonitors (NULL, NULL, enum_monitor, (LPARAM) &index);
-
-  _gdk_offset_x = G_MININT;
-  _gdk_offset_y = G_MININT;
-
-  /* Calculate offset */
-  for (i = 0; i < _gdk_num_monitors; i++)
-    {
-      _gdk_offset_x = MAX (_gdk_offset_x, -_gdk_monitors[i].rect.x);
-      _gdk_offset_y = MAX (_gdk_offset_y, -_gdk_monitors[i].rect.y);
-    }
-  GDK_NOTE (MISC, g_print ("Multi-monitor offset: (%d,%d)\n",
-			   _gdk_offset_x, _gdk_offset_y));
-
-  /* Translate monitor coords into GDK coordinate space */
-  for (i = 0; i < _gdk_num_monitors; i++)
-    {
-      _gdk_monitors[i].rect.x += _gdk_offset_x;
-      _gdk_monitors[i].rect.y += _gdk_offset_y;
-      GDK_NOTE (MISC, g_print ("Monitor %d: %dx%d@%+d%+d\n",
-			       i, _gdk_monitors[i].rect.width,
-			       _gdk_monitors[i].rect.height,
-			       _gdk_monitors[i].rect.x,
-			       _gdk_monitors[i].rect.y));
-    }
-}
-
 static LRESULT CALLBACK
 inner_display_change_window_procedure (HWND   hwnd,
                                        UINT   message,
@@ -261,7 +178,7 @@ inner_display_change_window_procedure (HWND   hwnd,
       {
         GdkWin32Display *win32_display = GDK_WIN32_DISPLAY (_gdk_display);
 
-        _gdk_monitor_init ();
+        _gdk_monitor_init (GDK_WIN32_SCREEN (win32_display->screen));
         _gdk_screen_init_root_window_size (GDK_WIN32_SCREEN (win32_display->screen));
         g_signal_emit_by_name (win32_display->screen, "size_changed");
 
@@ -349,7 +266,7 @@ _gdk_win32_display_open (const gchar *display_name)
 
   win32_display->screen = g_object_new (GDK_TYPE_WIN32_SCREEN, NULL);
 
-  _gdk_monitor_init ();
+  _gdk_monitor_init (GDK_WIN32_SCREEN (win32_display->screen));
   _gdk_visual_init (win32_display->screen);
   _gdk_screen_init_root_window (GDK_WIN32_SCREEN (win32_display->screen));
   _gdk_events_init ();
@@ -739,6 +656,8 @@ gdk_win32_display_finalize (GObject *object)
 {
   GdkWin32Display *display_win32 = GDK_WIN32_DISPLAY (object);
 
+  g_list_free (display_win32->input_devices);
+
   _gdk_win32_display_finalize_cursors (display_win32);
   _gdk_win32_dnd_exit ();
 
@@ -806,7 +725,7 @@ gdk_win32_display_class_init (GdkWin32DisplayClass *klass)
   display_class->supports_input_shapes = gdk_win32_display_supports_input_shapes;
   display_class->supports_composite = gdk_win32_display_supports_composite;
 
-  display_class->list_devices = _gdk_win32_display_list_devices;
+  display_class->list_devices = gdk_win32_display_list_devices;
   //? display_class->get_app_launch_context = _gdk_win32_display_get_app_launch_context;
   display_class->get_cursor_for_type = _gdk_win32_display_get_cursor_for_type;
   display_class->get_cursor_for_name = _gdk_win32_display_get_cursor_for_name;
