@@ -26,6 +26,10 @@ struct _GdkWin32Display
 {
   GdkDisplay display;
 
+  GdkScreen *screen;
+
+  GList *input_devices;
+
   Win32CursorTheme *cursor_theme;
   gchar *cursor_theme_name;
   int cursor_theme_size;
