@@ -257,16 +257,6 @@ extern LRESULT CALLBACK _gdk_win32_window_procedure (HWND, UINT, WPARAM, LPARAM)
 
 extern GdkDisplay       *_gdk_display;
 
-extern gint		 _gdk_num_monitors;
-typedef struct _GdkWin32Monitor GdkWin32Monitor;
-struct _GdkWin32Monitor
-{
-  gchar *name;
-  gint width_mm, height_mm;
-  GdkRectangle rect;
-};
-extern GdkWin32Monitor  *_gdk_monitors;
-
 /* Offsets to add to Windows coordinates (which are relative to the
  * primary monitor's origin, and thus might be negative for monitors
  * to the left and/or above the primary monitor) to get GDK
@@ -278,6 +268,8 @@ extern HDC		 _gdk_display_hdc;
 extern HINSTANCE	 _gdk_dll_hinstance;
 extern HINSTANCE	 _gdk_app_hmodule;
 
+extern gboolean		 _gdk_input_ignore_core;
+
 /* These are thread specific, but GDK/win32 works OK only when invoked
  * from a single thread anyway.
  */
@@ -545,13 +537,10 @@ void _gdk_win32_emit_configure_event (GdkWindow *window);
 void _gdk_win32_windowing_init (void);
 void _gdk_screen_init_root_window (GdkWin32Screen *screen_win32);
 void _gdk_screen_init_root_window_size (GdkWin32Screen *screen);
-void _gdk_monitor_init(void);
+void _gdk_monitor_init(GdkWin32Screen *screen);
 void _gdk_visual_init (GdkScreen *screen);
 void _gdk_dnd_init    (void);
 void _gdk_events_init (void);
-void _gdk_input_init  (GdkDisplay *display);
 void _gdk_input_wintab_init_check (GdkDeviceManager *device_manager);
 
-extern gboolean _is_win8_or_later;
-
 #endif /* __GDK_PRIVATE_WIN32_H__ */
