@@ -100,6 +100,12 @@ gtk_css_image_win32_parse (GtkCssImage  *image,
       return FALSE;
     }
 
+  if (! _gtk_css_parser_try (parser, ",", TRUE))
+    {
+      _gtk_css_parser_error (parser, "Expected ','");
+      return FALSE;
+    }
+
   if (!_gtk_css_parser_try_int (parser, &wimage->state))
     {
       _gtk_css_parser_error (parser, "Expected a valid integer value");
@@ -123,6 +129,12 @@ gtk_css_image_win32_parse (GtkCssImage  *image,
               return FALSE;
             }
 
+          if (! _gtk_css_parser_try (parser, ",", TRUE))
+            {
+              _gtk_css_parser_error (parser, "Expected ','");
+              return FALSE;
+            }
+
           if (!_gtk_css_parser_try_int (parser, &wimage->state2))
             {
               _gtk_css_parser_error (parser, "Expected a valid integer value");
