@@ -19,6 +19,12 @@
 
 #include "config.h"
 
+/* Vista or newer */
+#define _WIN32_WINNT 0x0600
+#define WINVER _WIN32_WINNT
+#define NTDDI_VERSION NTDDI_VISTA
+#define COBJMACROS
+
 #include "gtkfilechoosernativeprivate.h"
 #include "gtknativedialogprivate.h"
 
@@ -41,12 +47,6 @@
 #include "gtkfilechooserentry.h"
 #include "gtkfilefilterprivate.h"
 
-/* Vista or newer */
-#define _WIN32_WINNT 0x0600
-#define WINVER _WIN32_WINNT
-#define NTDDI_VERSION NTDDI_VISTA
-#define COBJMACROS
-
 #include "win32/gdkwin32.h"
 #include <shlobj.h>
 #include <windows.h>
@@ -182,7 +182,7 @@ ifiledialogevents_OnFolderChange (IFileDialogEvents *self,
     {
       events->got_hwnd = TRUE;
 
-      hr = IFileDialog_QueryInterface (pfd, &IID_IOleWindow, &olew);
+      hr = IFileDialog_QueryInterface (pfd, &IID_IOleWindow, (LPVOID *) &olew);
       if (SUCCEEDED (hr))
         {
           hr = IOleWindow_GetWindow (olew, &dialog_hwnd);
@@ -260,6 +260,7 @@ static IFileDialogEventsVtbl ifde_vtbl = {
   ifiledialogevents_OnOverwrite
 };
 
+static void
 file_dialog_events_send_close (IFileDialogEvents *self)
 {
   FileDialogEvents *events = (FileDialogEvents *)self;
@@ -367,7 +368,7 @@ get_shell_item_for_uri (const char *uri)
   HRESULT hr;
   gunichar2 *uri_w = g_utf8_to_utf16 (uri, -1, NULL, NULL, NULL);
 
-  hr = SHCreateItemFromParsingName(uri_w, 0, &IID_IShellItem, &item);
+  hr = SHCreateItemFromParsingName(uri_w, 0, &IID_IShellItem, (LPVOID *) &item);
   if (SUCCEEDED (hr))
     return item;
   else
@@ -396,11 +397,7 @@ filechooser_win32_thread (gpointer _data)
   HRESULT hr;
   IFileDialog *pfd = NULL;
   IFileDialog2 *pfd2 = NULL;
-  gboolean res = FALSE;
   DWORD flags;
-  HWND parent = NULL;
-  HWND dialog_hwnd;
-  GtkWindow *transient_for;
   DWORD cookie;
   GSList *l;
 
@@ -409,11 +406,11 @@ filechooser_win32_thread (gpointer _data)
   if (data->save && !data->folder)
     hr = CoCreateInstance (&CLSID_FileSaveDialog,
                            NULL, CLSCTX_INPROC_SERVER,
-                           &IID_IFileSaveDialog, &pfd);
+                           &IID_IFileSaveDialog, (LPVOID *) &pfd);
   else
     hr = CoCreateInstance (&CLSID_FileOpenDialog,
                            NULL, CLSCTX_INPROC_SERVER,
-                           &IID_IFileOpenDialog, &pfd);
+                           &IID_IFileOpenDialog, (LPVOID *) &pfd);
 
   if (FAILED (hr))
     g_error ("Can't create FileOpenDialog: %s\n", g_win32_error_message (hr));
@@ -465,7 +462,7 @@ filechooser_win32_thread (gpointer _data)
     {
       gunichar2 *label = g_utf8_to_utf16 (data->cancel_label, -1,
                                         NULL, NULL, NULL);
-      hr = IFileDialog_QueryInterface (pfd, &IID_IFileDialog2, &pfd2);
+      hr = IFileDialog_QueryInterface (pfd, &IID_IFileDialog2, (LPVOID *) &pfd2);
       if (SUCCEEDED (hr))
         {
           IFileDialog2_SetCancelButtonLabel (pfd2, label);
@@ -489,7 +486,7 @@ filechooser_win32_thread (gpointer _data)
   if (data->current_file)
     {
       IFileSaveDialog *pfsd;
-      hr = IFileDialog_QueryInterface (pfd, &IID_IFileSaveDialog, &pfsd);
+      hr = IFileDialog_QueryInterface (pfd, &IID_IFileSaveDialog, (LPVOID *) &pfsd);
       if (SUCCEEDED (hr))
         {
           IShellItem *item = get_shell_item_for_file (data->current_file);
@@ -545,7 +542,7 @@ filechooser_win32_thread (gpointer _data)
   if (SUCCEEDED (hr))
     {
       IFileOpenDialog *pfod = NULL;
-      hr = IFileDialog_QueryInterface (pfd,&IID_IFileOpenDialog, &pfod);
+      hr = IFileDialog_QueryInterface (pfd,&IID_IFileOpenDialog, (LPVOID *) &pfod);
 
       if (SUCCEEDED (hr))
         {
@@ -565,7 +562,7 @@ filechooser_win32_thread (gpointer _data)
               IShellItem *item;
               hr = IShellItemArray_GetItemAt (res, i, &item);
               if (FAILED (hr))
-                g_error ("Can't get item at %d: %s\n", i, g_win32_error_message (hr));
+                g_error ("Can't get item at %lu: %s\n", i, g_win32_error_message (hr));
               data_add_shell_item (data, item);
               IShellItem_Release (item);
             }
@@ -672,7 +669,6 @@ gtk_file_chooser_native_win32_show (GtkFileChooserNative *self)
   guint update_preview_signal;
   GSList *filters, *l;
   int n_filters, i;
-  COMDLG_FILTERSPEC *win32_filters;
 
   if (gtk_file_chooser_get_extra_widget (GTK_FILE_CHOOSER (self)) != NULL)
     return FALSE;
