@@ -42,25 +42,43 @@ struct _GtkIconHelperPrivate {
 
   guint use_fallback : 1;
   guint force_scale_pixbuf : 1;
+  guint rendered_surface_is_symbolic : 1;
 
   cairo_surface_t *rendered_surface;
 };
 
 G_DEFINE_TYPE_WITH_PRIVATE (GtkIconHelper, gtk_icon_helper, GTK_TYPE_CSS_GADGET)
 
-void
+static void
 gtk_icon_helper_invalidate (GtkIconHelper *self)
 {
   if (self->priv->rendered_surface != NULL)
     {
       cairo_surface_destroy (self->priv->rendered_surface);
       self->priv->rendered_surface = NULL;
+      self->priv->rendered_surface_is_symbolic = FALSE;
     }
 
   if (!GTK_IS_CSS_TRANSIENT_NODE (gtk_css_gadget_get_node (GTK_CSS_GADGET (self))))
     gtk_widget_queue_resize (gtk_css_gadget_get_owner (GTK_CSS_GADGET (self)));
 }
 
+void
+gtk_icon_helper_invalidate_for_change (GtkIconHelper     *self,
+                                       GtkCssStyleChange *change)
+{
+  GtkIconHelperPrivate *priv = self->priv;
+
+  if (change &&
+      ((gtk_css_style_change_affects (change, GTK_CSS_AFFECTS_SYMBOLIC_ICON) &&
+        priv->rendered_surface_is_symbolic) ||
+      (gtk_css_style_change_affects (change, GTK_CSS_AFFECTS_ICON) &&
+       !priv->rendered_surface_is_symbolic)))
+    {
+      gtk_icon_helper_invalidate (self);
+    }
+}
+
 static void
 gtk_icon_helper_take_definition (GtkIconHelper      *self,
                                  GtkImageDefinition *def)
@@ -145,8 +163,7 @@ static void
 gtk_icon_helper_style_changed (GtkCssGadget      *gadget,
                                GtkCssStyleChange *change)
 {
-  if (gtk_css_style_change_affects (change, GTK_CSS_AFFECTS_ICON))
-    gtk_icon_helper_invalidate (GTK_ICON_HELPER (gadget));
+  gtk_icon_helper_invalidate_for_change (GTK_ICON_HELPER (gadget), change);
 
   if (!GTK_IS_CSS_TRANSIENT_NODE (gtk_css_gadget_get_node (gadget)))
     GTK_CSS_GADGET_CLASS (gtk_icon_helper_parent_class)->style_changed (gadget, change);
@@ -205,6 +222,7 @@ gtk_icon_helper_init (GtkIconHelper *self)
 
   self->priv->icon_size = GTK_ICON_SIZE_INVALID;
   self->priv->pixel_size = -1;
+  self->priv->rendered_surface_is_symbolic = FALSE;
 }
 
 static void
@@ -419,6 +437,7 @@ ensure_surface_for_gicon (GtkIconHelper    *self,
                           gint              scale,
                           GIcon            *gicon)
 {
+  GtkIconHelperPrivate *priv = self->priv;
   GtkIconTheme *icon_theme;
   gint width, height;
   GtkIconInfo *info;
@@ -478,18 +497,20 @@ ensure_surface_for_gicon (GtkIconHelper    *self,
       symbolic = FALSE;
     }
 
+  surface = gdk_cairo_surface_create_from_pixbuf (destination, scale, gtk_widget_get_window (gtk_css_gadget_get_owner (GTK_CSS_GADGET (self))));
+
   if (!symbolic)
     {
       GtkCssIconEffect icon_effect;
 
-      surface = gdk_cairo_surface_create_from_pixbuf (destination, scale, gtk_widget_get_window (gtk_css_gadget_get_owner (GTK_CSS_GADGET (self))));
       icon_effect = _gtk_css_icon_effect_value_get (gtk_css_style_get_value (style, GTK_CSS_PROPERTY_ICON_EFFECT));
       gtk_css_icon_effect_apply (icon_effect, surface);
     }
   else
     {
-      surface = gdk_cairo_surface_create_from_pixbuf (destination, scale, gtk_widget_get_window (gtk_css_gadget_get_owner (GTK_CSS_GADGET (self))));
+      priv->rendered_surface_is_symbolic = TRUE;
     }
+
   g_object_unref (destination);
 
   return surface;
