@@ -718,7 +718,7 @@ initiates_touch_drag (GtkPaned *paned,
   GtkAllocation allocation;
 
 #define TOUCH_EXTRA_AREA_WIDTH 50
-  gtk_widget_get_allocation (GTK_WIDGET (paned), &allocation);
+  gtk_css_gadget_get_content_allocation (priv->gadget, &allocation, NULL);
   gtk_css_gadget_get_preferred_size (priv->handle_gadget,
                                      priv->orientation,
                                      -1,
@@ -761,7 +761,7 @@ gesture_drag_begin_cb (GtkGestureDrag *gesture,
   sequence = gtk_gesture_single_get_current_sequence (GTK_GESTURE_SINGLE (gesture));
   event = gtk_gesture_get_last_event (GTK_GESTURE (gesture), sequence);
   device = gdk_event_get_source_device (event);
-  gtk_widget_get_allocation (GTK_WIDGET (paned), &allocation);
+  gtk_css_gadget_get_content_allocation (priv->gadget, &allocation, NULL);
   paned->priv->panning = FALSE;
 
   is_touch = (event->type == GDK_TOUCH_BEGIN ||
@@ -1290,15 +1290,13 @@ gtk_paned_get_preferred_height_for_width (GtkWidget *widget,
 }
 
 static void
-flip_child (GtkWidget     *widget,
-            GtkAllocation *child_pos)
+flip_child (const GtkAllocation *allocation,
+            GtkAllocation       *child_pos)
 {
-  GtkAllocation allocation;
   gint x, width;
 
-  gtk_widget_get_allocation (widget, &allocation);
-  x = allocation.x;
-  width = allocation.width;
+  x = allocation->x;
+  width = allocation->width;
 
   child_pos->x = 2 * x + width - child_pos->x - child_pos->width;
 }
@@ -1424,9 +1422,9 @@ gtk_paned_allocate (GtkCssGadget        *gadget,
 
           if (gtk_widget_get_direction (GTK_WIDGET (widget)) == GTK_TEXT_DIR_RTL)
             {
-              flip_child (widget, &(window2_allocation));
-              flip_child (widget, &(window1_allocation));
-              flip_child (widget, &(priv->handle_pos));
+              flip_child (allocation, &(window2_allocation));
+              flip_child (allocation, &(window1_allocation));
+              flip_child (allocation, &(priv->handle_pos));
             }
 
           child1_allocation.x = child1_allocation.y = 0;
@@ -1628,7 +1626,7 @@ gtk_paned_create_child_window (GtkPaned  *paned,
                                          NULL, &handle_size,
                                          NULL, NULL);
 
-      gtk_widget_get_allocation (widget, &allocation);
+      gtk_css_gadget_get_content_allocation (priv->gadget, &allocation, NULL);
       if (priv->orientation == GTK_ORIENTATION_HORIZONTAL &&
           child == priv->child2 && priv->child1 &&
           gtk_widget_get_visible (priv->child1))
@@ -1798,41 +1796,42 @@ gtk_paned_render (GtkCssGadget *gadget,
   GtkWidget *widget = gtk_css_gadget_get_owner (gadget);
   GtkPaned *paned = GTK_PANED (widget);
   GtkPanedPrivate *priv = paned->priv;
-  GtkStyleContext *context = gtk_widget_get_style_context (widget);
-
-  if (gtk_cairo_should_draw_window (cr, priv->child1_window))
-    {
-      cairo_save (cr);
-      gtk_cairo_transform_to_window (cr, widget, priv->child1_window);
-      gtk_render_background (context,
-                             cr,
-                             0, 0,
-                             gdk_window_get_width (priv->child1_window),
-                             gdk_window_get_height (priv->child1_window));
-      cairo_restore (cr);
-    }
-
-  if (gtk_cairo_should_draw_window (cr, priv->child2_window))
-    {
-      cairo_save (cr);
-      gtk_cairo_transform_to_window (cr, widget, priv->child2_window);
-      gtk_render_background (context,
-                             cr,
-                             0, 0,
-                             gdk_window_get_width (priv->child2_window),
-                             gdk_window_get_height (priv->child2_window));
-      cairo_restore (cr);
-    }
+  GtkAllocation widget_allocation;
+  int window_x, window_y;
 
+  gtk_widget_get_allocation (widget, &widget_allocation);
   if (gtk_cairo_should_draw_window (cr, gtk_widget_get_window (widget)) &&
       priv->child1 && gtk_widget_get_visible (priv->child1) &&
       priv->child2 && gtk_widget_get_visible (priv->child2))
+    gtk_css_gadget_draw (priv->handle_gadget, cr);
+
+  if (priv->child1 && gtk_widget_get_visible (priv->child1))
     {
-      gtk_css_gadget_draw (priv->handle_gadget, cr);
+      gdk_window_get_position (priv->child1_window, &window_x, &window_y);
+      cairo_save (cr);
+      cairo_rectangle (cr, 
+                       window_x - widget_allocation.x,
+                       window_y - widget_allocation.y,
+                       gdk_window_get_width (priv->child1_window),
+                       gdk_window_get_height (priv->child1_window));
+      cairo_clip (cr);
+      gtk_container_propagate_draw (GTK_CONTAINER (widget), priv->child1, cr);
+      cairo_restore (cr);
     }
 
-  /* Chain up to draw children */
-  GTK_WIDGET_CLASS (gtk_paned_parent_class)->draw (widget, cr);
+  if (priv->child2 && gtk_widget_get_visible (priv->child2))
+    {
+      gdk_window_get_position (priv->child2_window, &window_x, &window_y);
+      cairo_save (cr);
+      cairo_rectangle (cr, 
+                       window_x - widget_allocation.x,
+                       window_y - widget_allocation.y,
+                       gdk_window_get_width (priv->child2_window),
+                       gdk_window_get_height (priv->child2_window));
+      cairo_clip (cr);
+      gtk_container_propagate_draw (GTK_CONTAINER (widget), priv->child2, cr);
+      cairo_restore (cr);
+    }
 
   return FALSE;
 }
@@ -1983,14 +1982,13 @@ update_drag (GtkPaned *paned,
 {
   GtkPanedPrivate *priv = paned->priv;
   GtkAllocation allocation;
-  GtkWidget *widget = GTK_WIDGET (paned);
   gint pos;
   gint handle_size;
   gint size;
   gint x, y;
 
   gdk_window_get_position (priv->handle, &x, &y);
-  gtk_widget_get_allocation (widget, &allocation);
+  gtk_css_gadget_get_content_allocation (priv->gadget, &allocation, NULL);
   if (priv->orientation == GTK_ORIENTATION_HORIZONTAL)
     pos = xpos;
   else
