@@ -72,7 +72,7 @@
  * # CSS nodes
  *
  * |[<!-- language="plain" -->
- * progressbar
+ * progressbar[.osd]
  * ╰── trough
  *     ├── [text]
  *     ╰── progress[.pulse]
@@ -83,6 +83,8 @@
  * is shown. The progress subnode has the style class .pulse when in activity
  * mode. It gets the style classes .left, .right, .top or .bottom added when
  * the progress 'touches' the corresponding end of the GtkProgressBar.
+ * The .osd class on the progressbar node is for use in overlays like the one
+ * epiphany has for page loading progress.
  */
 
 #define MIN_HORIZONTAL_BAR_WIDTH   150
@@ -503,11 +505,11 @@ update_node_state (GtkProgressBar *pbar)
 
   state = gtk_widget_get_state_flags (GTK_WIDGET (pbar));
 
-  gtk_css_node_set_state (gtk_css_gadget_get_node (priv->gadget), state);
-  gtk_css_node_set_state (gtk_css_gadget_get_node (priv->trough_gadget), state);
-  gtk_css_node_set_state (gtk_css_gadget_get_node (priv->progress_gadget), state);
+  gtk_css_gadget_set_state (priv->gadget, state);
+  gtk_css_gadget_set_state (priv->trough_gadget, state);
+  gtk_css_gadget_set_state (priv->progress_gadget, state);
   if (priv->text_gadget)
-    gtk_css_node_set_state (gtk_css_gadget_get_node (priv->text_gadget), state);
+    gtk_css_gadget_set_state (priv->text_gadget, state);
 }
 
 static void
